
;---------------------------------------------------------------------------
;                recode mask and get a buffer zone of cloud and shadow
;              
;                            Coded by Xiaolin Zhu
;           Department of ecosystem science and sustainability
;                       Colorado state university
;                                2015-11-30
;                     Contact: zhuxiaolin55@gmail.com
;                  *All copyright belongs to Xiaolin Zhu*
;----------------------------------------------------------------------------

;open the image

Pro GetData,ImgData = ImgData,ns = ns,nl = nl,nb = nb,Data_Type = Data_Type,$
    FileName = FileName,Map_info = map_Info, Fid = Fid
    Filter = ['all file;*.*']
    Envi_Open_File,FileName,R_Fid = Fid
    Envi_File_Query,Fid,ns = ns,nl = nl,nb = nb,Data_Type = Data_Type
    map_info = envi_get_map_info(fid=Fid)
    dims = [-1,0,ns - 1 ,0,nl - 1]
    case Data_Type Of
        1:ImgData = BytArr(ns,nl,nb)    ;  BYTE  Byte
        2:ImgData = IntArr(ns,nl,nb)    ;  INT  Integer
        3:ImgData = LonArr(ns,nl,nb)    ;  LONG  Longword integer
        4:ImgData = FltArr(ns,nl,nb)    ;  FLOAT  Floating point
        5:ImgData = DblArr(ns,nl,nb)    ;  DOUBLE  Double-precision floating
        6:ImgData = COMPLEXARR(ns,nl,nb); complex, single-precision, floating-point
        9:ImgData = DCOMPLEXARR(ns,nl,nb);complex, double-precision, floating-point
        12:ImgData = UINTARR(ns,nl,nb)   ; unsigned integer vector or array
        13:ImgData = ULONARR(ns,nl,nb)   ;  unsigned longword integer vector or array
        14:ImgData = LON64ARR(ns,nl,nb)   ;a 64-bit integer vector or array
        15:ImgData = ULON64ARR(ns,nl,nb)   ;an unsigned 64-bit integer vector or array
    EndCase
    For i = 0,nb-1 Do Begin
       Dt = Envi_Get_Data(Fid = Fid,dims = dims,pos=i)
       ImgData[*,*,i] = Dt[*,*]
    EndFor
End


;*******************************************************************
;                         main program
;*******************************************************************

pro  buffer_cloud

;set the following parameters
  buffer_zone=2    ;width of buffer
 

 t0=systime(1)                  ;Initial time


;open final cloud mask time-series

    FileName0 = Dialog_PickFile(title = 'open ATSA mask images:')   ; shadow 0, clear 1, cloud 2, gap/background 3
     GetData,ImgData=mask,ns = ns,nl = nl,nb = nb,Data_Type = Data_Type,FileName = FileName0,Fid = Fid1
     
     mask2=mask
     ;recode the pixel values for running missing pixel repairing code: NSPI_TIMESERIES.pro , clear 0, gap/background 1 shadow 2 cloud 3 buffer 4
     for ib=0, nb-1, 1 do begin
      ind0=where(mask[*,*,ib] eq 0, num_0)
      if (num_0 gt 0) then begin
          temp=mask2[*,*,ib]
          temp[ind0]=2
          mask2[*,*,ib]=temp
      endif
      ind1=where(mask[*,*,ib] eq 1, num_1)
      if (num_1 gt 1) then begin
        temp=mask2[*,*,ib]
        temp[ind1]=0
        mask2[*,*,ib]=temp
      endif
      ind2=where(mask[*,*,ib] eq 2, num_2)
      if (num_2 gt 0) then begin
        temp=mask2[*,*,ib]
        temp[ind2]=3
        mask2[*,*,ib]=temp
      endif
      ind3=where(mask[*,*,ib] eq 3, num_3)
      if (num_3 gt 0) then begin
        temp=mask2[*,*,ib]
        temp[ind3]=1
        mask2[*,*,ib]=temp
      endif
        
     endfor
     
     ;get buffer
     mask_b=mask2
     for i=0,ns-1,1 do begin
       for j=0,nl-1,1 do begin
         for ii=0,nb-1, 1 do begin
           
           ;get the buffer zone
           if (mask2[i,j,ii] eq 3 or mask2[i,j,ii] eq 2 ) then begin  ;identify cloud and shadow
             a1=max([0, i-buffer_zone])
             a2=min([ns-1, i+buffer_zone])
             b1=max([0, j-buffer_zone])
             b2=min([nl-1, j+buffer_zone])
             mask_win=mask2[a1:a2,b1:b2,ii]
             ind_c=where(mask_win eq 0, num_c)
             if (num_c gt 0) then begin
               temp=mask_b[a1:a2,b1:b2,ii]
               temp[ind_c]=4
               mask_b[a1:a2,b1:b2,ii]=temp
             endif
           endif
         endfor
       endfor
     endfor

     ;;output the results
   
     FileName =FileName0+'_recoded_buffered'
     Envi_Write_Envi_File,mask_b,Out_Name = FileName,Map_info = map_Info




print, 'time used', floor((systime(1)-t0)/3600), 'h',floor(((systime(1)-t0) mod 3600)/60),'m',(systime(1)-t0) mod 60,'s'


end